@@ -19,6 +19,7 @@ class Player(IntEnum):
 
 
 class InvalidMovesMessage(str, Enum):
+    INVALID_STATUS_CODE = "Bot is supposed to return status code 200. Received invalid status code"
     OUT_OF_BOUNDS = "Given co-ordinates are out of bound. All Co-ordinates should be between 0 and 4"
     INVALID_DIRECTION = "Pieces cannot move backwards. Player 0 moves up the board and player 1 moves down"
     INVALID_SOURCE = "Invalid Source. Source co-ordinates must have one of the players piece on it"
@@ -60,3 +61,10 @@ class BotResponse(BaseModel):
 class PlayerMove(BaseModel):
     move: Move
     player: Player
+
+
+class GameResult(str, Enum):
+    PLAYER1_WINS = "player1_wins"
+    PLAYER2_WINS = "player2_wins"
+    DRAW = "draw"
+    INVALID = "invalid"
