@@ -10,24 +10,26 @@ from colorama import Fore, Style
 from .schema import Piece, Move, Player, InvalidMovesMessage, BotResponse, PlayerMove
 from .custom_exceptions import InvalidMovesException, TimeoutException, MovesExceededException
 from .constants import MAX_MOVES_ALLOWED
+from .logger_utils import get_logger
 
 logging.basicConfig()
 
 
 class Simulator:
-    def __init__(self, move_timeout: int = 4, verbose: bool = True) -> None:
+    def __init__(self, move_timeout: int = 4, verbose: bool = True, logging_level: int = logging.WARNING) -> None:
         pieces = ['B', 'b', '_', 'r', 'R']
         self.board = [[pieces[i] for _ in range(5)] for i in range(5)]
         self.RED_PIECES = [Piece.RED_BOMBER, Piece.RED_STINGER]
         self.BLUE_PIECES = [Piece.BLUE_BOMBER, Piece.BLUE_STINGER]
         self.move_timeout = move_timeout
-        self.logger = logging.getLogger(__name__)
-        self.logger.setLevel(level=logging.INFO)
-        if not verbose:
-            self.logger.setLevel(level=logging.WARNING)
+        self.logger = get_logger(__name__)
+        self.logger.setLevel(level=logging_level)
         self.verbose = verbose
 
     def print_board(self):
+        """
+            Print Board state while testing bots in local development
+        """
         if self.verbose:
             print(Fore.BLACK + Style.BRIGHT + "  0 1 2 3 4")
             for index, row in enumerate(self.board):
@@ -52,7 +54,22 @@ class Simulator:
             raise InvalidMovesException(
                 reason=InvalidMovesMessage.INVALID_DIRECTION, player=player)
 
-    def validate_endpoints(self, move: Move, player: Player):
+    def validate_endpoints(self, move: Move, player: Player) -> None:
+        """
+        Check if the source and destination endpoints are valid. 
+        Source should hold a piece of the player and destination should be an empty space
+        Raise an exception if this is not followed, return None otherwise
+
+        Parameters
+        ----------
+        move : Move
+        player : Player
+
+        Raises
+        ------
+        InvalidMovesException
+        InvalidMovesException
+        """
         if self.board[move.dst.x][move.dst.y] != Piece.EMPTY_SPACE:
             raise InvalidMovesException(
                 reason=InvalidMovesMessage.INVALID_DESTINATION, player=player)
@@ -63,8 +80,9 @@ class Simulator:
         ):
             raise InvalidMovesException(
                 reason=InvalidMovesMessage.INVALID_SOURCE, player=player)
+        return None
 
-    def validate_elimination_move(self, move: Move, player: Player):
+    def validate_elimination_move(self, move: Move, player: Player) -> None:
         if move.src.x == move.dst.y == 0 or move.src.x == move.src.y == 4:
             raise InvalidMovesException(
                 reason=InvalidMovesMessage.HOME_ROW_ELIMINATION_FORBIDDEN, player=player)
@@ -77,7 +95,7 @@ class Simulator:
         ) or (
             player.BLUE and eliminated_piece in self.RED_PIECES
         ):
-            return
+            return None
         else:
             raise InvalidMovesException(
                 reason=InvalidMovesMessage.INVALID_ELIMINATION_TARGET, player=player)
@@ -85,7 +103,7 @@ class Simulator:
     def validate_stinger_move(self, move: Move, player: Player):
         if abs(move.src.x - move.dst.x) == abs(move.src.y - move.dst.y) == 1:
             # simply diagonal move (direction in y axis has already been validated)
-            return
+            return None
         elif (
             # horizontal elimination
             abs(move.src.x - move.dst.x) == 2 and move.src.y == move.dst.y
@@ -95,6 +113,7 @@ class Simulator:
         ):
             self.validate_elimination_move(move, player)
         else:
+            # move doesn't fit either simple moves or elimination moves
             raise InvalidMovesException(
                 reason=InvalidMovesMessage.INVALID_STINGER_MOVE, player=player)
 
@@ -113,7 +132,16 @@ class Simulator:
             raise InvalidMovesException(
                 reason=InvalidMovesMessage.INVALID_BOMBER_MOVE, player=player)
 
-    def validate_move(self, move: Move, player: Player):
+    def validate_move(self, move: Move, player: Player) -> None:
+        """
+        Check if the move is valid for the given board state and given Player
+        Raise the required exception if move is invalid, return None otherwise
+        Parameters
+        ----------
+        move : Move
+        player : Player
+        """
+        self.logger.info("Validating ")
         self.validate_direction(move, player)
         self.validate_endpoints(move, player)
         piece = self.board[move.src.x][move.src.y]
@@ -123,6 +151,14 @@ class Simulator:
             self.validate_bomber_move(move, player)
 
     def make_move(self, move: Move, player: Player):
+        """
+        Make the given move and Modify the board state accordingly
+
+        Parameters
+        ----------
+        move : Move
+        player : Player
+        """
         self.board[move.dst.x][move.dst.y] = self.board[move.src.x][move.src.y]
         self.board[move.src.x][move.src.y] = Piece.EMPTY_SPACE
         if abs(move.src.x - move.dst.x) == 2 or abs(move.src.y - move.dst.y) == 2:
@@ -149,29 +185,36 @@ class Simulator:
             }
             response = requests.post(
                 bot_urls[player], json=bot_input, timeout=self.move_timeout)
+            if response.status_code != 200:
+                self.logger.error(
+                    f"Invalid stauts code {response.status_code} received from player {player}. Expected: {200}")
+                raise InvalidMovesException(
+                    InvalidMovesMessage.INVALID_STATUS_CODE, player=player)
             try:
                 bot_response = BotResponse.validate(
                     json.loads(response.content))
                 bot_data[player] = bot_response.data
                 self.validate_move(bot_response.move, player=player)
-                self.logger.info(f"Move by Player: {player}")
-                self.logger.info(bot_response.move)
+                if self.verbose:
+                    print(f"Player {player} move")
+                    print(bot_response.move)
                 self.make_move(bot_response.move, player=player)
                 self.print_board()
                 game_over, winner = self.check_if_game_over()
                 if game_over:
-                    self.logger.info(f"Player {player} wins")
+                    if self.verbose:
+                        print(f"Player {player} wins")
                     return winner
                 player_moves.append(PlayerMove(
                     move=bot_response.move, player=player))
-                player ^= player  # infers type of player as int
-                # functions expect call of type Player Enum
+                player ^= player  # toggles player but pylance infers type of player as int
+                # functions expect call of type Player Enum. Adding this cast to silence pylance errors
                 player = Player(player)
             except InvalidMovesException as imex:
                 self.logger.error(
                     f"Invalid response received from player: {player}. Message: ", imex.message)
                 raise InvalidMovesException(
-                    reason=imex.reason, player=player)  # append player info
+                    reason=imex.reason, player=player)  # append player info to exception
             except Timeout as t:
                 self.logger.error(
                     f"Request to player: {player} times out")
